/*********
 * Hackathon Lumiere 6 & 7 dec 2021
 * Le Cube ruban LED + ecran Oled 1.3"
 * 
  Rui Santos
  Complete project details at https://RandomNerdTutorials.com/esp8266-nodemcu-web-server-slider-pwm/
  
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files.
  
  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.
// https://www.wikidebrouillard.org/wiki/Utiliser_le_D1_mini_avec_Arduino
https://www.wikidebrouillard.org/wiki/Item:D1_mini
*********/

// Importer les bibliothèque dont on a besoin

#include <ESP8266WiFi.h>
#include <ESPAsyncTCP.h>
#include <ESPAsyncWebServer.h>
#include <Adafruit_NeoPixel.h>  // ruban LED https://www.wiki.lesfabriquesduponant.net/index.php?title=Papifab_2020
#include <Adafruit_Sensor.h>
//#include <Adafruit_GFX.h>
//#include <Adafruit_SSD1306.h>
#include <NTPClient.h>    // https://randomnerdtutorials.com/esp32-ntp-client-date-time-arduino-ide/
#include <WiFiUdp.h>
#include"SH1106.h"        // WEMOS https://www.teachmemicro.com/1-3-i2c-oled-arduino-esp8266-tutorial/

// definition des couleurs https://www.rapidtables.org/fr/web/color/RGB_Color.html
// Logo icone image
#include "sieste.h"
#include "activite.h"
#include "arrivee.h"
#include "gouter.h"
#include "repas.h"
#include "depart.h"
#include "rangement.h"

//#define DEMO    // commenter pour passer en mode normal
#define DUREE 2 // duree creneau en mode demo
#define INTENS 120  // intensite du ruban LED (0-255)
#define LED_BLANCHE D0  // diametre 10mm, seuil 2.6v, R=100 => i=5.5mA
#define SCREEN_WIDTH 128  // largeur ecran
#define SCREEN_HEIGHT 64  // hauteur ecran
#define broche    D4 // Broche du ruban de led
#define nbpixels 30 // nombre de led sur le ruban

// Replace with your network credentials
const char* ssid = "PAPI";
const char* password = "";

const int output = 2;

String sliderValue = "500";
const char* PARAM_INPUT = "value";

unsigned long epoch = 0;
int nujour = 0; //numero jour de la semaine avec 0 pour dimanche
String jour = "mon jour"; // dimanche, lundi, etc.
String heure = "mon heure ..";
String date = "ma date ..";
int heure_n;
int minute_n;

const int nb_creneaux = 14; // 2 hors cloche et 12 dans la journee
int progress = 0;     // pour barre de progression
int n_creneau = 0;    // creneau actuel
int n_creneau_pre = 0;    // creneau precedent
int minute_jour = 0;  // minute courante
// struture creneau horaire
struct CRENEAU{
  int heure;
  uint32_t couleur;
  String message;
};
struct CRENEAU creneau[nb_creneaux+1]; 

// Initialize the OLED display using Arduino Wire: 
SH1106 display(0x3C,D1,D2); //WEMOS attention inversion branchement par rapport au tuto

// Create AsyncWebServer object on port 80
AsyncWebServer server(80);

//Creation objet WIFI UDP puis du client NTP
WiFiUDP ntpUDP;
// Avec intervalle de mise à jour de 60 sec ...
// ... et offset en secondes cad decalage horaire, pour GMT+1 mettre 3600, GMT +8 mettre 28800, etc.
// ... et le serveur NTP utilise est : europe.pool.ntp.org
NTPClient timeClient(ntpUDP, "europe.pool.ntp.org", 3600, 60000);

// page internet
const char index_html[] PROGMEM = R"rawliteral(
<!DOCTYPE HTML><html>
<head>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <title>Pilote ton ruban de LED</title>
  <style>
    html {font-family: Arial; display: inline-block; text-align: center;}
    h2 {font-size: 2.3rem;}
    p {font-size: 1.9rem;}
    body {max-width: 400px; margin:0px auto; padding-bottom: 25px;}
    .slider { -webkit-appearance: none; margin: 14px; width: 360px; height: 25px; background: #FFD65C;
      outline: none; -webkit-transition: .2s; transition: opacity .2s;}
    .slider::-webkit-slider-thumb {-webkit-appearance: none; appearance: none; width: 35px; height: 35px; background: #003249; cursor: pointer;}
    .slider::-moz-range-thumb { width: 35px; height: 35px; background: #003249; cursor: pointer; } 
  </style>
</head>
<body>
  <h2>Pilote Le Cube</h2>
  <p><span id="textSliderValue">%SLIDERVALUE%</span></p>
  <p><input type="range" onchange="updateSliderPWM(this)" id="pwmSlider" min="0" max="1023" value="%SLIDERVALUE%" step="1" class="slider"></p>
<script>
function updateSliderPWM(element) {
  var sliderValue = document.getElementById("pwmSlider").value;
  document.getElementById("textSliderValue").innerHTML = sliderValue;
  console.log(sliderValue);
  var xhr = new XMLHttpRequest();
  xhr.open("GET", "/slider?value="+sliderValue, true);
  xhr.send();
}
</script>
</body>
</html>
)rawliteral";

// Replaces placeholder with button section in your web page
String processor(const String& var){
  //Serial.println(var);
  if (var == "SLIDERVALUE"){
    return sliderValue;
  }
  return String();
}

// Lors de la configuration de la bibliothèque NeoPixel, 
// nous lui indiquons combien de pixels,
// et quelle broche utiliser pour envoyer des signaux.
Adafruit_NeoPixel strip(nbpixels, broche, NEO_GRB + NEO_KHZ800);
// pour configurer le ruban :
// nomDuRuban (nombreDePixels, NumeroDeLaBroche, typeDeRuban)
// Les différents types de ruban :
//   NEO_KHZ800  800 KHz bitstream (la plupart des produits Neopixel ou références de LED w/WS2812)
//   NEO_KHZ400  400 KHz (classic 'v1' (pas v2) FLORA pixels, drivers WS2811)
//   NEO_GRB     Pixels cablés pour GRB bitstream (la plupart des produits Neopixel)
//   NEO_RGB     Pixels cablés pour RGB bitstream (v1 FLORA pixels, pas les v2)
//   NEO_RGBW    Pixels cablés pour RGBW bitstream (produits NeoPixel RGBW)

int choix = 0;

void conf_cube() {
  // definition des couleurs  https://www.rapidtables.org/fr/web/color/RGB_Color.html
  const uint32_t noir = strip.Color(0, 0, 0);  // noir
  const uint32_t blanc = strip.Color(255, 255, 255);  // blanc
  const uint32_t jaune = strip.Color(255, 255, 0);  // jaune
  const uint32_t rouge = strip.Color(255, 0, 0);  // rouge
  const uint32_t vert = strip.Color( 0, 255, 0);  // vert
  const uint32_t bleu = strip.Color( 0, 0, 255);  // bleu
  const uint32_t cyan = strip.Color( 0, 255, 255);  // cyan
  const uint32_t orange = strip.Color( 255, 180, 100);  // cyan

  // definition des creneaux : heure, couleur
  creneau[0] = {0, noir, "Hors cloche"};
  creneau[1] = {7*60+30, jaune, "Accueil matin"};
  creneau[2] = {9*60+30, blanc, "Rangement"};
  creneau[3] = {9*60+45, rouge, "Activités"};
  creneau[4] = {11*60+30, blanc, "Rangement"};
  creneau[5] = {11*60+45, vert, "Repas"};
  creneau[6] = {13*60, blanc, "Rassemblement"};
  creneau[7] = {13*60+20, bleu, "Sieste"};
  creneau[8] = {15*60+30, cyan, "Lever Sieste"};
  creneau[9] = {15*60+45, blanc, "Rassemblement"};
  creneau[10] = {16*60, vert, "Goûter"};
  creneau[11] = {16*60+30, blanc, "Temps libre"};
  creneau[12] = {17*60, blanc, "Accueil soir"};
  creneau[13] = {19*60, noir, "Hors cloche"};

  // affichage de "Bonjour"
  display.setFont(ArialMT_Plain_24);
  display.setTextAlignment(TEXT_ALIGN_CENTER);
  display.drawString(64, 0, "Bonjour");

// affichage de "la journée commence bien avec le Cube"
  display.setFont(ArialMT_Plain_16);
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  display.drawStringMaxWidth(0, 30, 128,"La journée commence bien avec le Cube." );

// affichage de "Adresse IP"
  display.setTextAlignment(TEXT_ALIGN_CENTER);
  display.drawString(64, 50,  WiFi.localIP().toString());
  display.display();
  delay (5000);
  display.clear();
}
/*
 * configuration du mode demo : on definit des créneaux de n minutes
 */
void conf_demo(){
  // Recup heure 
  timeClient.update();
  heure = timeClient.getFormattedTime(); // heure  
  minute_jour = timeClient.getHours() * 60 + timeClient.getMinutes(); // minute courante de la journée    
  creneau[1].heure = minute_jour+DUREE; 
  int i=0;
  for (i = 2; i < nb_creneaux; i++) {
    creneau[i].heure = creneau[i-1].heure + DUREE;
  }
}
/*
 * setup general
 */
void setup(){
  // allumage led blanche Resistance 220 ohm
  pinMode(LED_BLANCHE, OUTPUT);
  digitalWrite(LED_BLANCHE, HIGH);
 
  // Serial port for debugging purposes
  Serial.begin(115200);

  analogWrite(output, sliderValue.toInt());

  // Connect to Wi-Fi
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("Connecting to WiFi..");
  }

  // Print ESP Local IP Address
  Serial.println(WiFi.localIP());

  // Route for root / web page
  server.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/html", index_html, processor);
  });

  // Send a GET request to <ESP_IP>/slider?value=<inputMessage>
  server.on("/slider", HTTP_GET, [] (AsyncWebServerRequest *request) {
    String inputMessage;
    // GET input1 value on <ESP_IP>/slider?value=<inputMessage>
    if (request->hasParam(PARAM_INPUT)) {
      inputMessage = request->getParam(PARAM_INPUT)->value();
      sliderValue = inputMessage;
      analogWrite(output, sliderValue.toInt());
    }
    else {
      inputMessage = "No message sent";
    }
    Serial.println(inputMessage);
    request->send(200, "text/plain", "OK");
  });
  
  // Start server
  server.begin();
  // Demarrage client NTP
  timeClient.begin();

  //démarrage du ruban de led
  strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.show();            // Turn OFF all pixels ASAP
  strip.setBrightness(INTENS); // Set BRIGHTNESS to about 1/5 (max = 255)
  // Initialising the display.
  display.init();

  conf_cube();  // config variables
#ifdef DEMO
  conf_demo();  // config mode demo (petits creneaux) pour test
#endif
} // end setup()

/*
 * Boucle principale
 */
void loop() {
  choix = sliderValue.toInt();
  choix = map(choix, 0, 1023, 50, 250); // Nb niveaux d'intensite des led
  strip.setBrightness(choix); // Set BRIGHTNESS to about 1/5 (max = 255)

  // clear the display buffer
  display.clear();

    // Recup heure puis affichage
  timeClient.update();
//    nujour = timeClient.getDay();    // jour de la semaine 0=dimanche
  heure = timeClient.getFormattedTime(); // heure  
  minute_jour = timeClient.getHours() * 60 + timeClient.getMinutes(); // minute de la journée    
   
    // Envoi des donnees recuperees sur la liaison serie
  Serial.print(" - heure : ");
  Serial.print(heure);
  Serial.print(" - min_jour : ");
  Serial.println(minute_jour);
// detection du creneau horaire actuel
  int i;
  if ((minute_jour < creneau[1].heure) || (minute_jour > creneau[nb_creneaux-1].heure) ) { // hors cloche
    n_creneau = 0;
  } else 
    for (i = 1; i < nb_creneaux-1; i++) {
      if ((creneau[i].heure <= minute_jour ) && (minute_jour < creneau[i+1].heure ) ){
        n_creneau = i;
      } 
  }
  Serial.print("--creneau : ");
  Serial.println(n_creneau);
  // affiche couleur ruban led en fonction du creneau
  colorWipe(creneau[n_creneau].couleur, 5);
  
//  if (n_creneau != n_creneau_pre) { // si changement de creneau on change l'image
//    n_creneau_pre = n_creneau;
    switch (n_creneau) { // affichage des images
      case 0:
      {
        Serial.println("Hors cloche");
        // affichage de "Hors cloche"
        display.setFont(ArialMT_Plain_24);
        display.setTextAlignment(TEXT_ALIGN_CENTER);
        display.drawString(64, 0, "Hors cloche");
        display.drawString(65, 32, heure);
        display.display();
        delay (25);
        display.clear();
      }
      break;
      case 1:
      {
        Serial.println("Accueil matin");
        // affiche image  
        display.drawXbm(45,0, arrivee_width, arrivee_height, arrivee_bits);
      }
      break;
      // tous les creneaux Rangement/Rassemblement
      case 2:
      case 4:
      case 6:
      case 8:
      case 9:
      {
        Serial.println("Rangement/Rassemblement");
        display.drawXbm(48,0, rangement_width, rangement_height, rangement_bits);
      }
      break;
      case 3:
      {
        Serial.println("Activité");
        display.drawXbm(50,0, activite_width, activite_height, activite_bits);
      }
      break;
      case 5:
      {
        Serial.println("Repas");
        display.drawXbm(50,0, repas_width, repas_height, repas_bits);
      }
      break;
      case 7:
      {
        Serial.println("Sieste");
        display.drawXbm(45,0, sieste_width, sieste_height, sieste_bits);   
      }
      break;
      case 10:
      {
        Serial.println("Gouter");
        display.drawXbm(45,0, gouter_width, gouter_height, gouter_bits);
      }
      break;
      case 11:
      {
        Serial.println("Temps libre");
        display.drawXbm(50,0, activite_width, activite_height, activite_bits);
      }
      break;
      case 12:
      {
        Serial.println("Accueil soir");
        display.drawXbm(35,0, depart_width, depart_height, depart_bits);
      }
      break;
    }
//  }
  // progression dans le creneau en % 
  if (n_creneau > 0) {
    progress = (minute_jour - creneau[n_creneau].heure) * 100 / (creneau[n_creneau+1].heure - creneau[n_creneau].heure);
    Serial.print("--progression : ");
    Serial.println(progress);
    // affiche la barre de progression
    display.drawProgressBar(2, 52, 124, 10, progress);
//    display.setTextAlignment(TEXT_ALIGN_CENTER);
    display.setTextAlignment(TEXT_ALIGN_LEFT);
    display.setFont(ArialMT_Plain_10);
//    display.drawString(65, 32, String(progress) + "%");
    display.drawString(2, 35, heure + " - " + creneau[n_creneau].message);
    display.display();
    delay (25);
    display.clear();
  }

}

// Some functions of our own for creating animated effects -----------------

// Fill strip pixels one after another with a color. Strip is NOT cleared
// first; anything there will be covered pixel by pixel. Pass in color
// (as a single 'packed' 32-bit value, which you can get by calling
// strip.Color(red, green, blue) as shown in the loop() function above),
// and a delay time (in milliseconds) between pixels.
void colorWipe(uint32_t color, int wait) {
  for(int i=0; i<strip.numPixels(); i++) { // For each pixel in strip...
    strip.setPixelColor(i, color);         //  Set pixel's color (in RAM)
    strip.show();                          //  Update strip to match
    delay(wait);                           //  Pause for a moment
  }
}
